/* MDDRIVER.C - test driver for MD2, MD4 and MD5 *//* Copyright (C) 1990-2, RSA Data Security, Inc. Created 1990. Allrights reserved.RSA Data Security, Inc. makes no representations concerning eitherthe merchantability of this software or the suitability of thissoftware for any particular purpose. It is provided "as is"without express or implied warranty of any kind.These notices must be retained in any copies of any part of thisdocumentation and/or software. *//* The following makes MD default to MD5 if it has not already been  defined with C compiler flags. */#ifndef MD#ifndef OSK#define MD MD5#else#define MD 5#endif#endif#include <stdio.h>#ifdef OSK#include "/dd/blarsdefs/time.h"#include "/dd/blarsdefs/string.h"#else#include <time.h>#include <string.h>#endif#include "global.h"#ifdef MD2#include "md2.h"#endif#ifdef MD4#include "md4.h"#endif#ifdef MD5#include "md5.h"#endif/* Length of test block, number of test blocks. */#define TEST_BLOCK_LEN 1000#define TEST_BLOCK_COUNT 1000static void MDString();static void MDTimeTrial();static void MDTestSuite();static void MDFile();static void MDFilter();static void MDPrint();#ifdef MD2#define MD_CTX MD2_CTX#define MDInit MD2Init#define MDUpdate MD2Update#define MDFinal MD2Final#endif#ifdef MD4#define MD_CTX MD4_CTX#define MDInit MD4Init#define MDUpdate MD4Update#define MDFinal MD4Final#endif#ifdef MD5#define MD_CTX MD5_CTX#define MDInit MD5Init#define MDUpdate MD5Update#define MDFinal MD5Final#endif/* Main driver.Arguments (may be any combination):  -sstring - digests string  -t       - runs time trial  -x       - runs test script  filename - digests file  (none)   - digests standard input */int main(argc, argv)int argc;char *argv[];{  int i;  if(argc > 1) for(i = 1; i < argc; i++)   if(argv[i][0] == '-' && argv[i][1] == 's')     MDString(argv[i] + 2);   else if(strcmp(argv[i], "-t") == 0)     MDTimeTrial();   else if(strcmp(argv[i], "-x") == 0)     MDTestSuite();#ifdef OSK   else if(strcmp(argv[i],"-h") == 0){   fprintf(stderr,"\nMD%d - Calculate a Message Digest Fingerprint (Checksum)", MD);   fprintf(stderr,"\n   Usage: MD%d <-opts> <filename>\n", MD);   fprintf(stderr,"    Opts:  -sstring - digests string\n");   fprintf(stderr,"           -t       - runs time trial\n");   fprintf(stderr,"           -x       - runs test script\n");   fprintf(stderr,"           -h       - this help file\n");   fprintf(stderr," filename: digests file\n");   fprintf(stderr,"  (none) : digests standard input\n");}#endif   else     MDFile(argv[i]);  else MDFilter();  return(0);}/* Digests a string and prints the result. */static void MDString(string)char *string;{  MD_CTX context;  unsigned char digest[16];  unsigned int len = strlen(string);  MDInit(&context);  MDUpdate(&context, string, len);  MDFinal(digest, &context);  printf("MD%d (\"%s\") = ", MD, string);  MDPrint(digest);  printf("\n");}/* Measures the time to digest TEST_BLOCK_COUNT TEST_BLOCK_LEN-byte  blocks. */static void MDTimeTrial(){  MD_CTX context;  time_t endTime, startTime;  unsigned char block[TEST_BLOCK_LEN], digest[16];  unsigned int i;  printf("\nMD%d time trial. Digesting %d %d-byte blocks ...", MD, TEST_BLOCK_LEN, TEST_BLOCK_COUNT);#ifdef OSKfflush(stdout);#endif  /* Initialize block */  for(i = 0; i < TEST_BLOCK_LEN; i++) block[i] = (unsigned char)(i & 0xff);  /* Start timer */  time(&startTime);  /* Digest blocks */  MDInit(&context);  for(i = 0; i < TEST_BLOCK_COUNT; i++) MDUpdate(&context, block, TEST_BLOCK_LEN);  MDFinal(digest, &context);  /* Stop timer */  time(&endTime);  printf(" done\n");  printf("Digest = ");  MDPrint(digest);  printf("\nTime = %ld seconds\n", (long)(endTime-startTime));  /*   * Be careful that endTime-startTime is not zero.   * (Bug fix from Ric Anderson, ric@Artisoft.COM.)   */  printf("Speed = %ld bytes/second\n",\  (long)TEST_BLOCK_LEN * (long)TEST_BLOCK_COUNT/((endTime-startTime) != 0 ? (endTime-startTime):1));}/* Digests a reference suite of strings and prints the results. */static void MDTestSuite(){  printf("MD%d test suite:\n", MD);  MDString("");  MDString("a");  MDString("abc");  MDString("message digest");  MDString("abcdefghijklmnopqrstuvwxyz");  MDString("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789");  MDString("1234567890123456789012345678901234567890\1234567890123456789012345678901234567890");}/* Digests a file and prints the result. */static void MDFile(filename)char *filename;{  FILE *file;  MD_CTX context;  int len;  unsigned char buffer[1024], digest[16];#ifndef OSK  if((file = fopen(filename, "rb")) == NULL) printf("%s can't be opened\n", filename);#else  if((file = fopen(filename, "r")) == NULL) printf("%s can't be opened\n", filename);#endif  else { MDInit(&context); while(len = fread (buffer, 1, 1024, file))   MDUpdate(&context, buffer, len); MDFinal(digest, &context); fclose(file); printf("MD%d (%s) = ", MD, filename); MDPrint(digest); printf("\n");  }}/* Digests the standard input and prints the result. */static void MDFilter(){  MD_CTX context;  int len;  unsigned char buffer[16], digest[16];  MDInit(&context);  while(len = fread (buffer, 1, 16, stdin)) MDUpdate(&context, buffer, len);  MDFinal(digest, &context);  MDPrint(digest);  printf("\n");}/* Prints a message digest in hexadecimal. */static void MDPrint(digest)unsigned char digest[16];{  unsigned int i;  for(i = 0; i < 16; i++) printf("%02x", digest[i]);}